@@ -1,6 +1,6 @@
 package Net::Statsd;
-BEGIN {
-  $Net::Statsd::VERSION = '0.01';
+{
+  $Net::Statsd::VERSION = '0.02';
 }
 
 # ABSTRACT: Sends statistics to the stats daemon over UDP
@@ -8,6 +8,7 @@ BEGIN {
 
 use strict;
 use warnings;
+use Carp ();
 use IO::Socket ();
 
 our $HOST = 'localhost';
@@ -18,6 +19,10 @@ our $PORT = 8125;
 sub timing {
     my ($stat, $time, $sample_rate) = @_;
 
+    if (! defined $sample_rate) {
+        $sample_rate = 1;
+    }
+
     my $stats = {
         $stat => sprintf "%d|ms", $time
     };
@@ -47,8 +52,13 @@ sub decrement {
 sub update_stats {
     my ($stats, $delta, $sample_rate) = @_;
 
-    $delta = 1 unless defined $delta;
-    $sample_rate = 1 unless defined $sample_rate;
+    if (! defined $delta) {
+        $delta = 1;
+    }
+
+    if (! defined $sample_rate) {
+        $sample_rate = 1;
+    }
 
     if (! ref $stats) {
         $stats = [ $stats ];
@@ -63,20 +73,41 @@ sub update_stats {
 }
 
 
-sub send {
+sub _sample_data {
     my ($data, $sample_rate) = @_;
 
-    my %sampled_data;
+    my $sampled_data;
+
+    if (! $data || ref $data ne 'HASH') {
+        Carp::croak("No data?");
+    }
+
+    if (! defined $sample_rate) {
+        $sample_rate = 1;
+    }
 
     if ($sample_rate < 1) {
         if (rand() <= $sample_rate) {
             while (my ($stat, $value) = each %{ $data }) {
-                $sampled_data{$stat} = sprintf "%s|@%s", $value, $sample_rate;
+                $sampled_data->{$stat} = sprintf "%s|@%s", $value, $sample_rate;
             }
         }
     }
+
     else {
-        %sampled_data = %{ $data };
+        $sampled_data = $data;
+    }
+
+    return $sampled_data;
+}
+
+
+sub send {
+    my ($data, $sample_rate) = @_;
+
+    my $sampled_data = _sample_data($data, $sample_rate);
+    if (! $sampled_data) {
+        Carp::croak("No (sampled) data to be sent?");
     }
 
     my $udp_sock = IO::Socket::INET->new(
@@ -92,8 +123,8 @@ sub send {
 
     my $all_sent = 1;
 
-    for my $stat (keys %sampled_data) {
-        my $value =$data->{$stat};
+    for my $stat (keys %{ $sampled_data }) {
+        my $value = $data->{$stat};
         my $packet = "$stat:$value";
         $udp_sock->send($packet);
         # XXX If you want warnings...
@@ -125,7 +156,7 @@ Net::Statsd - Sends statistics to the stats daemon over UDP
 
 =head1 VERSION
 
-version 0.01
+version 0.02
 
 =head1 SYNOPSIS
 
@@ -233,7 +264,23 @@ equivalent to:
 A sampling rate less than 1 means only update the stats
 every x number of times (0.1 = 10% of the times).
 
-=head2 C<send(\%data, $sample_rate=1)>
+=head2 C<_sample_data(\%data, $sample_rate = 1)>
+
+B<This method is used internally, it's not part of the public interface.>
+
+Takes care of transforming a hash of metrics data into
+a B<sampled> hash of metrics data, according to the given
+C<$sample_rate>.
+
+If C<$sample_rate == 1>, then sampled data is exactly the
+incoming data.
+
+If C<$sample_rate = 0.2>, then every metric value will be I<marked>
+with the given sample rate, so the Statsd server will automatically
+scale it. For example, with a sample rate of 0.2, the metric values
+will be multiplied by 5.
+
+=head2 C<send(\%data, $sample_rate = 1)>
 
 Squirt the metrics over UDP.
 
@@ -245,7 +292,7 @@ Cosimo Streppone <cosimo@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2011 by Cosimo Streppone.
+This software is copyright (c) 2012 by Cosimo Streppone.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
